import time
import math
import os
import datetime

from generic_module import Module
from collections import deque
from lxml import etree
import threading

class AggregationModule(Module):
  def __init__(self, name, matching_timeout, fuzziness, save_to_disk, boss_receive, root_path):
    Module.__init__(self, name, root_path)
    self.matching_timeout = matching_timeout
    self.fuzziness = fuzziness
    self.save_to_disk = save_to_disk
    self.timeout = 0
    self.boss_receive = boss_receive
    
    self.vehicles_dict = dict()
    self.deque_lock = threading.Lock()
    self.offsets_from_boss_dict = dict()
  
  def tell_offset(self, recv_name, offsets_dict):
    self.offsets_from_boss_dict[recv_name] = offsets_dict
    self.log(str(self.offsets_from_boss_dict))
  
  def add_vehicle(self, vehicle, recv_module_name):
    sub_vehicle = vehicle.getchildren()[0]
    #in case of no lane data, lane = 0 (no actual lane will ever have the index 0)
    lane = "0"
    try:
      lane = sub_vehicle.find("lane").text
    except Exception, ex:
      self.log(str(ex))
      pass
    #print etree.tostring(sub_vehicle)
    if recv_module_name not in self.vehicles_dict:
      self.vehicles_dict[recv_module_name] = dict()
    if lane not in self.vehicles_dict[recv_module_name]:
      self.vehicles_dict[recv_module_name][lane] = deque()
    with self.deque_lock:
      self.vehicles_dict[recv_module_name][lane].append(sub_vehicle)
        
  #--- "private"
  def find_and_form(self, boss_receive_sub_veh):
    ts = boss_receive_sub_veh.find("ts").text
    start = time.mktime(datetime.datetime.strptime(ts, "%Y-%m-%d-%H-%M-%S-%f").timetuple())
    #---
    vehicle = etree.Element("vehicle")
    vehicle.append(boss_receive_sub_veh)
    #clear "dead" recvs
    
    #boss_lane
    lane = "0"
    try:
      lane = boss_receive_sub_veh.find("lane").text
    except Exception:
      pass
    #print self.vehicles_dict
    for key in self.vehicles_dict:
      if key == self.boss_receive:
        continue
      try:
        with self.deque_lock:
          while "0" in self.vehicles_dict[key] and len(self.vehicles_dict[key]["0"]) > 0 and self.compare_timestamps(self.vehicles_dict[key]["0"][0], boss_receive_sub_veh, key) == 3:
            self.vehicles_dict[key]["0"].popleft()
          while lane != "0" and lane in self.vehicles_dict[key] and len(self.vehicles_dict[key][lane]) > 0 and self.compare_timestamps(self.vehicles_dict[key][lane][0], boss_receive_sub_veh, key) == 3:
            self.vehicles_dict[key][lane].popleft()
      except Exception, ex:
        self.log("Find and form error 1.")
        try:
          self.deque_lock.release()
        except Exception:
          pass
        return

    found_list = list()
    found_list.append(self.boss_receive)
    #---
    #if both boss receive and guest receive event are on the same lane, use that
    #if either boss, or guest (don't), don't have lane defined, use lane "0"
    while True:
      if len(found_list) == len(self.vehicles_dict):
        #broken = True
        break
      
      for key, value in self.vehicles_dict.iteritems():
        if key not in found_list:
          try:
            with self.deque_lock:
              if lane in value:
                #self.log(str(key) + " " + str(lane) + " " + str(len(value[lane])))
                for i in range(len(value[lane])):
                  #self.log(str(value[lane]))
                  if self.compare_timestamps(value[lane][i], boss_receive_sub_veh, key) == 1:
                    vehicle.append(value[lane][i])
                    found_list.append(key)
                    del value[lane][i]
                    break
              elif lane != "0" and "0" in value and lane not in value:
                for i in range(len(value["0"])):
                  if self.compare_timestamps(value["0"][i], boss_receive_sub_veh, key) == 1:
                    vehicle.append(value["0"][i])
                    found_list.append(key)
                    del value["0"][i]
                    #broken = True
                    break
          except Exception, ex:
            self.log("Find and form error 2.")
            self.log(str(ex))
            try:
              self.deque_lock.release()
            except Exception:
              pass
            break
      if time.time() - start > self.matching_timeout:
        break
      self.zzzzz(0.5)
    return vehicle   
    
  def save_to_file(self, str_vehicle):
    #print "start save"
    if self.sname == "":
      return 1
    first_line = "<swd><site><name>" + self.sname + "</name><vehicles>\n"
    last_line = "</vehicles></site></swd>"
    try:
      date = datetime.datetime.now().strftime('%Y-%m-%d')
      if not os.path.exists(self.sites_path + "/" + self.sname + "/ext"):
        os.mkdir(self.sites_path + "/" + self.sname + "/ext")
      file = open(self.sites_path + "/" + self.sname + "/ext/" + date + ".xml", "a+")
      #remove last line          
      file.seek(0, os.SEEK_END)
      pos = file.tell() - 1
      while pos > 0 and file.read(1) != "\n":
        pos -= 1
        file.seek(pos, os.SEEK_SET)
      if pos > 0:
        file.seek(pos, os.SEEK_SET)
        file.truncate()
      #if file is empty, add first line
      if self.sname != "" and pos == file.tell() - 1:
        file.write(first_line + str_vehicle + "\n\n" + last_line)
      else:
        file.write(str_vehicle + "\n" + last_line)
      file.close()
    except Exception, ex:
      try:
        file.close()
      except Exception:
        pass
      self.log(str(ex))
    #print "end save"
  
  #module_name1 is the name of the "guest" module    
  def compare_timestamps(self, sub_veh1, sub_veh2, module_name1):
    try:
      ts1 = sub_veh1.find("ts").text
      ts2 = sub_veh2.find("ts").text
      v = 0
      try:
        v = float(sub_veh2.find("v").text)
      except Exception:
        try:
          v = float(sub_veh1.find("v").text)
        except Exception:
          pass
   
      l1 = sub_veh1.find("lane")
      add_to_offset = sub_veh1.find("siwim_i_add_to_offset")
  
      parts1 = ts1.split("-")
      parts2 = ts2.split("-")
  
      t_offset = 0
      factor = 1
      
      #timestamp1 = time.mktime(datetime.datetime.strptime(ts1, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts1[len(parts1) - 1]) / 1000 + t_offset
      #timestamp2 = time.mktime(datetime.datetime.strptime(ts2, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts2[len(parts2) - 1]) / 1000
      
      #self.log("original comark: " + ts1)
      #self.log("original siwim: " + ts2)
      
      try:
        t_offset = self.offsets_from_boss_dict[module_name1][l1.text] / v
        #self.log("offset 1 " + str(t_offset))
        try:
          factor = self.offsets_from_boss_dict[module_name1][l1.text] / abs(self.offsets_from_boss_dict[module_name1][l1.text])
          t_offset = t_offset + (float(add_to_offset.text) * factor)
          #self.log("offset 2 " + str(t_offset))
        except Exception, ex:
          self.log(str(ex))
      except Exception, ex:
        self.log(str(ex))
      
      timestamp1 = time.mktime(datetime.datetime.strptime(ts1, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts1[len(parts1) - 1]) / 1000 
      timestamp2 = time.mktime(datetime.datetime.strptime(ts2, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts2[len(parts2) - 1]) / 1000 + t_offset
      
      #self.log("comparing comark " + str(timestamp1) + " with siwim " + str(timestamp2))
      
      #self.log(str(timestamp1))
      #self.log(str(timestamp2))
      
      #self.log("----------------")
      
      if math.fabs(timestamp1 - timestamp2) <= self.fuzziness:
        self.log("fuzz: " + str(self.fuzziness))
        self.log("matched " + ts1 + " with " + ts2)
        self.log(str(t_offset))
        return 1
      else:
        if timestamp1 > timestamp2:
          return 2
        else:
          return 3
    except Exception, ex:
      self.log("Compare timestamps error.")
      self.log(str(ex))
      return 3
            
  def run(self):
    self.alive = True
    self.end = False
    #self.log("Starting " + self.name + " aggregation module ...")
    try:
      while True:
        if self.end:
          self.alive = False
          return   
        #8 lanes fapp
        for i in range(9):
          lane = str(i)
          if self.boss_receive in self.vehicles_dict and lane in self.vehicles_dict[self.boss_receive] and len(self.vehicles_dict[self.boss_receive][lane]) > 0:
            boss_receive_sub_veh = self.vehicles_dict[self.boss_receive][lane].popleft()
            vehicle = self.find_and_form(boss_receive_sub_veh)
            if self.save_to_disk == 1:
              vehicle_str = etree.tostring(vehicle)
              self.save_to_file(vehicle_str)
            #print etree.tostring(vehicle)
            
            if vehicle.find("alpr") != None or vehicle.find("adr") != None:
              anpr_node = etree.Element("anpr")
              if vehicle.find("alpr") != None:
                anpr_node.append(vehicle.find("alpr").find("ts"))
                anpr_node.append(vehicle.find("alpr").find("lp"))
                vehicle.find("alpr").getparent().remove(vehicle.find("alpr"))
              if vehicle.find("adr") != None:
                anpr_node.append(vehicle.find("adr").find("hgp"))
                vehicle.find("adr").getparent().remove(vehicle.find("adr"))
              vehicle.append(anpr_node)
              
            for  key, mod in self.downstream_modules_dict.iteritems():
              mod.add_vehicle(vehicle, self.name)
              mod.set_site_name(self.sname)
          #print etree.tostring(vehicle)
        time.sleep(0.5)
    except Exception, ex:
      self.log(str(ex))
      self.alive = False
