import urllib
import os
import httplib
import time
import datetime
import ssl
from sys import platform

if platform == "win32":
  from ctypes import c_ulonglong, windll, c_wchar_p, pointer
elif platform == "linux" or platform == "linux2":
  import subprocess
  pass

from generic_module import Module

class AlarmModule(Module):
  def __init__(self, name, poll_rate, min_u, u_tresh, min_dd, interval_start, interval_end, timeouts, alarm_timeout, host, url, protocol, root_path):
    Module.__init__(self, name, root_path)
    self.name = name
    self.poll_rate = poll_rate
    self.min_u = min_u
    self.u_tresh = u_tresh
    self.min_dd = min_dd
    self.interval_start = interval_start
    self.interval_end = interval_end
    self.timeouts = timeouts
    self.alarm_timeout = alarm_timeout
    
    self.host = host
    self.sname = ""
    
    self.url = url
    self.protocol = protocol
    
    self.voltage = 0
    
    self.receive_xml_last_received = dict()
  
  def add_vehicle(self, vehicle, recv_module_name): 
    sub_vehicle = vehicle.getchildren()[0]
    self.name
    if (len(self.receive_xml_last_received) == 0) or (recv_module_name in self.receive_xml_last_received):
      lane = "a"
      try:
        lane = sub_vehicle.find("lane").text
      except Exception:
        pass
      if lane != "a":
        self.receive_xml_last_received[recv_module_name + ":" + str(lane)] = time.time()
      else:
        self.receive_xml_last_received[recv_module_name] = time.time()
        
  def getUniXhostname(self):
    sp = subprocess.Popen(["hostname"], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    return output.strip()
        
  def register_alarm(self, module_name):
    for i in range(9):
      if i == 0:
        self.receive_xml_last_received[module_name] = time.time()
      else:
        self.receive_xml_last_received[module_name + ":" + str(i)] = time.time()
        
  def set_site_name(self, sname):
    self.sname = sname
    
  def w32DiskFreeMB(self, p):
    lpFreeBytesAvailable = c_ulonglong(0)
    lpTotalNumberOfBytes = c_ulonglong(0)
    lpTotalNumberOfFreeBytes = c_ulonglong(0)
    windll.kernel32.GetDiskFreeSpaceExW(c_wchar_p(p), pointer(lpFreeBytesAvailable), pointer(lpTotalNumberOfBytes), pointer(lpTotalNumberOfFreeBytes))
    return lpTotalNumberOfFreeBytes.value / 1024 / 1024
  
  def uniX32PartitionFreeMB(self, mp):
    sp = subprocess.Popen(["df", "-B", "MB", mp], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    size = output.split("\n")[1].split()[3]
    return int(size[:size.find("M")])
    
  #def ageCTU(self):
    #fn = 'd:/siwim_mkiii/siwim_e/log/ctu_' + datetime.now().strftime('%Y-%m-%d') + '.txt'
    #if not os.path.exists(fn):
      #return '?'
    #try:
      #f = open(fn,'r')
      #return f.readlines()[-1].split('\t')[1][:4]
    #finally:
      #f.close()
  def set_voltage(self, voltage):
    self.voltage = voltage
  
  def alarm(self, code, msg, params):
    self.log("Firing an alarm, message: " + msg) 
    #print "lel"
    payload = None
    comp_name = ""
    if platform == "win32":
      comp_name = os.getenv('COMPUTERNAME').lower()
    elif platform == "linux" or platform == "linux2":
      comp_name = self.getUniXhostname().lower()
    if len(params) == 0:
      payload = urllib.urlencode({'sys':comp_name, 'type':code, 'msg':msg})
    if len(params) == 1:
      payload = urllib.urlencode({'sys':comp_name, 'type':code, 'msg':msg, 'par1':params[0]})
    if len(params) == 2:
      payload = urllib.urlencode({'sys':comp_name, 'type':code, 'msg':msg, 'par1':params[0], 'par2':params[1]})
    if len(params) > 2:
      payload = urllib.urlencode({'sys':comp_name, 'type':code, 'msg':msg, 'par1':params[0], 'par2':params[1], 'par3':params[2]})
    #print payload
    headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
    if self.protocol == "https":
      conn = httplib.HTTPSConnection(self.host, context=ssl._create_unverified_context(), timeout=10)
    elif self.protocol == "http":
      conn = httplib.HTTPConnection(self.host, timeout=10)
    conn.request('POST', self.url, payload, headers)
    resp = conn.getresponse().read()
    self.log(resp)
        
  def run(self):
    self.alive = True
    self.end = False
    traffic_sent = {}
    traffic_sent_at = {}
    voltage_sent = False 
    space_sent = False
    in_start = self.interval_start.split(":")
    in_end = self.interval_end.split(":")
    while True:
      if self.end:
        self.alive = False
        return   
      try:
        params1 = list()
        params2 = list()
        try:
          to_send_type1 = ""
          to_send_type2 = ""
          sep1 = ""
          sep2 = ""
          #print self.receive_xml_last_received
          for name, t in self.receive_xml_last_received.iteritems():
            #check if name is in traffic_sent_at dict, and decide whether to arm the alarm
            if name in traffic_sent_at and (time.time() - traffic_sent_at[name] > self.alarm_timeout): 
              traffic_sent[name] = False
            #if alarm isn't armed (true), disregard
            #print traffic_sent
            if name in traffic_sent and traffic_sent[name]: 
              continue
            #find lane info, 0 for default
            idx = "0"
            if ":" in name:
              parts = name.split(":")
              idx = parts[1]
            #if the index isn't listed in the conf file, disregard this iteration
            if idx not in self.timeouts:
              continue 
            #decide whether to use timeout or implicit timeout, based on current time
            tmt = float(self.timeouts[idx][1])
            if (datetime.datetime.now().hour > int(in_start[0]) or (datetime.datetime.now().hour == int(in_start[0]) and datetime.datetime.now().minute >= int(in_start[1]))) and (datetime.datetime.now().hour < int(in_end[0]) or (datetime.datetime.now().hour == int(in_end[0]) and datetime.datetime.now().minute <= int(in_end[1]))):
              tmt = float(self.timeouts[idx][0])
            #check if the alarm should be triggered; start building the alarm
            if time.time() - t > tmt:
              try:
                if idx != "0":
                  if sep1 == "":
                    params1.append(parts[0])
                    params1.append(idx)
                    params1.append(str(time.time() - t))
                  else:
                    params1[0] = params1[0] + sep1 + parts[0]
                    params1[1] = params1[1] + sep1 + idx
                    params1[2] = params1[2] + sep1 + str(time.time() - t)
                  to_send_type1 = to_send_type1 + sep1 + '*no ' + parts[0] + ' data* on lane ' + idx + ' for the last ' + str(time.time() - t) + ' seconds'
                  sep1 = ","
                else:
                  if sep2 == "":
                    params2.append(name)
                    params2.append(str(time.time() - t))
                  else:
                    params2[0] = params2[0] + sep2 + name
                    params2[1] = params2[1] + sep2 + str(time.time() - t)
                  to_send_type2 = to_send_type2 + sep2 + '*no ' + name + ' data* for the last ' + str(time.time() - t) + ' seconds'
                  sep2 = ","
                traffic_sent[name] = True
                traffic_sent_at[name] = time.time()
              except Exception, ex:
                self.log(str(ex))
          #if alarm string isn't empty, send the alarm
          if to_send_type1 != "":  
            self.alarm(4, to_send_type1, params1)
          if to_send_type2 != "":  
            self.alarm(5, to_send_type2, params2)
        except Exception, ex:
          self.log(str(ex))
        
        params = list()            
        try:
          u = float(self.voltage) 
          #print self.min_u
          if u > 0 and self.min_u > u:
            if not voltage_sent:
              voltage_sent = True
              try:
                params.append(str(self.min_u))
                self.alarm(2, 'Voltage below ' + str(self.min_u) + ' V.', params)
              except Exception, ex:
                self.log("u: " + str(ex))
          elif u > self.u_tresh:
            voltage_sent = False
        except Exception, ex:
          self.log(str(ex))
          pass
        
        params = list()
        try:
          if platform == "win32":
            dd = float(self.w32DiskFreeMB("d:"))
          else:
            dd = self.uniX32PartitionFreeMB(self.root_path)
          if self.min_dd > dd:
            if not space_sent:
              space_sent = True
              try:
                params.append(str(self.min_dd))
                self.alarm(3, 'D disk space below ' + str(self.min_dd) + ' MB.', params)
              except Exception, ex:
                self.log("dd: " + str(ex))
          else:
            space_sent = False
        except Exception, ex:
          pass
        
        self.zzzzz(self.poll_rate)
      except Exception, ex:
        self.log(str(ex))
        self.alive = False