import serial
import time
import socket
#import datetime
#import os
from sys import platform
import subprocess

from generic_module import Module

class CtuModule(Module):
  def __init__(self, name, udp_ip, udp_port, serial_port, baud, interval, vpn_host, vpn_port, root_path, relevant_for_listeners):
    Module.__init__(self, name, root_path)
    self.udp_ip = udp_ip
    self.udp_port = udp_port
    
    self.serial_port = serial_port
    self.baud = baud
    self.interval = interval
    
    self.vpn_host = vpn_host
    self.vpn_port = vpn_port
    
    self.relevant_for_listeners = relevant_for_listeners #=u, t, ut, 0
  
  def ctucmd(self, ser, cmnd, resp = '', to = 0.5):
    if ser.timeout != to:
      ser.timeout = to
    ser.flushInput()
    ser.write(cmnd + '\n')
    r = []
    while True:
      l = ser.readline().rstrip();
      if l == '': # EOF
        return r 
      r.append(l)
      if len(resp) > 0 and l.find(resp) != -1: # resp found
        return r
      if len(r) > 10: # safeguard
        return r
  
  def dict2confstr(self, section, d):
    r = '[' + section + ']' + '\n'
    for k,v in d.iteritems():
      r += k + '=' + v + '\n'
    return r
  
  def run(self):
    self.alive = True
    self.end = False
    even_odd_flag = 1
    try:
      ser = serial.Serial(self.serial_port, self.baud)
      try:
        while True:
          if self.end:
            ser.close()
            self.alive = False
            return
          #a temporary hack!
          if even_odd_flag == 0:
            even_odd_flag = 1
          else:
            even_odd_flag = 0
          #sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
          #result = sock.connect_ex((self.vpn_host, self.vpn_port))
          #try:
            #sock.close()
          #except Exception:
            #pass
          if platform == "win32":
            ping = subprocess.Popen(["ping", "-n", "1", "-w", "2", self.vpn_host],stdout = subprocess.PIPE).communicate()[0]
          else:
            ping = subprocess.Popen(["ping", "-c", "1", "-w", "2", self.vpn_host],stdout = subprocess.PIPE).communicate()[0]
          if ('unreachable' not in str(ping)) and ('timed' not in str(ping)) and ('failure' not in str(ping)): 
            self.ctucmd(ser, 'vPNON')
          d = {}
          d['dev'] = 'CTU-03';
          d['ts'] = time.strftime('%Y-%m-%d-%H-%M-%S')
          r = self.ctucmd(ser, 'FWVER')
          if len(r) == 1:
            j = r[0].find('firmware ver') 
            if j != -1:
              d['fw'] = r[0][j+12:].rstrip()
          r = self.ctucmd(ser, 'SENS')
          for i in range(len(r)):
            s = r[i].split('=')
            if len(s) == 2:
              j = -1
              u = '__?'
              if s[0].startswith('U') and "u" in self.relevant_for_listeners.lower():
                j = s[1].find('V')
                u = '__V'
              if s[0].startswith('T') and "t" in self.relevant_for_listeners.lower():
                j = s[1].find('C')
                u = '__C'
              if j != -1:
                d[s[0]+u] = s[1][:j]
          """
          try:
            if even_odd_flag == 0:
              line = d['ts'] + "\t" + d['Ub__V'] + "\t" + d['Ti__C']
              for i in range(1, 6):
                line = line + "\t" + d['T' + str(i) + "__C"]
              line = line + "\n"
              fd = open(self.root_path + "/siwim_e/log/ctu_" +  datetime.datetime.today().strftime('%Y-%m-%d') + ".txt", "a")
              fd.write(line)
              fd.close()
          except Exception, ex:
            self.log(str(ex))
          """
          
          for key, mod in self.downstream_modules_dict.iteritems():
            try:
              mod.set_ctu_fw(d['fw'])
            except Exception, ex:
              pass
            try:
              mod.set_voltage(d['Ub__V'])
            except Exception, ex:
              pass
          #if 'T1__C' in d:
          msg = self.dict2confstr('mydevice', d) #json.dumps(d)
          #self.log(msg) #DEBUG
          sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
          try:
            sock.sendto(msg, (self.udp_ip, self.udp_port))
          finally:
            sock.close()
          self.zzzzz(self.interval)
      finally:
        ser.close()
    except Exception, ex:
      self.log(str(ex))
      self.alive = False

